#!/usr/bin/env python
###################################
#
# Requires cris_viirs binary to work, set location in CRIS_VIIRS_BIN
#
###################################

import numpy as np
import numpy.ma as ma

from pyhdf.SD import *  #FIXME: don't import *
import h5py as h5

import subprocess
from intercal.hs2modis import Cris2Modis

import logging
log = logging.getLogger(__name__)

from config import *


def cris2modis_translate(scrisfile, nchan):
  scris = h5.File(scrisfile, 'r')
  sw = scris['All_Data/CrIS-SDR_All/ES_RealSW'][:]
  mw = scris['All_Data/CrIS-SDR_All/ES_RealMW'][:]
  lw = scris['All_Data/CrIS-SDR_All/ES_RealLW'][:]
  scris.close()

  c2m = Cris2Modis(SRF_ANCIL)
  rad, bt = c2m(nchan, lw, mw, sw)
  return rad, bt


def apply_mapping(mapping, data):
  datamapped = data[mapping[:,:,0], mapping[:,:,1], mapping[:,:,2]]
  datamasked = ma.array(datamapped, mask=mapping.mask[:,:,0], fill_value=-999.0)
  return datamasked.filled()


def cris2viirs_create_mapping(xTracks, yTracks, nfovs):
  mapping = np.empty((768,3200,3), dtype=np.int)  # FIXME: should mask or use NaNs
  mapping.fill(-1)

  scanlines, scanangles, detectors = nfovs.shape
  
  for s in range(scanlines):
    for a in range(scanangles):
      for d in range(detectors):
  
        nfov = nfovs[s][a][d]
        if nfov == 0:
          continue
  
        xs = xTracks[s][a][d]
        ys = yTracks[s][a][d]
  
        idxs = np.column_stack((ys, xs))
        for fov in range(nfov):
          yx = idxs[fov]
          y = yx[0] - 1
          x = yx[1] - 1

          mapping[y][x][0] = s
          mapping[y][x][1] = a
          mapping[y][x][2] = d
#          mapping[y][x] = np.array([s,a,d])

  mmapping = ma.masked_equal(mapping, -1)
  ma.set_fill_value(mmapping, 0)
  return mmapping


def cris2viirs_transform(gcrsofile, gmtcofile):
  cvfile = "cris-viirs-collo.hdf"

  cmd = "%s %s %s %s" % (CRIS_VIIRS_BIN, gcrsofile, gmtcofile, cvfile)
  log.debug(cmd)
  subprocess.check_call(cmd, shell=True)
  
  # Grab the important bits out of the created file
  cv = SD(cvfile, SDC.READ)
  xTracks = cv.select('VIIRS_Across_Track_IDX')[:]   # max 3200
  yTracks = cv.select('VIIRS_Along_Track_IDX')[:]    # max 768
  nfovs = cv.select('Number_Of_VIIRS_FOV')[:]
  cv.end()

  # Get rid of the cris_viirs file, we don't need it anymore
  cmd = "rm %s" % (cvfile)
  log.debug(cmd)
  subprocess.check_call(cmd, shell=True)

  # FIXME: MOVE THESE ONCE WE KNOW WHAT WE THINK OF NUMBA!
  
  mapping = cris2viirs_create_mapping(xTracks, yTracks, nfovs)

  return mapping



if __name__ == "__main__":
  import sys
  print sys.argv
  _, gcrso, scris, gmtco = sys.argv
  
  print " --- CALLING CRIS2VIIRS TRANSFORM"
  mapping = cris2viirs_transform(gcrso, gmtco)

  print "CALLING CRIS2MODIS TRANSLATE"
  rad31, bt31 = cris2modis_translate(scris, 31)

  vrad31 = apply_mapping(mapping, rad31)
  vbt31 = apply_mapping(mapping, bt31)

  def plot(arr, outname):
    import matplotlib.pyplot as plt
    plt.imshow(arr)
    plt.gray()
    plt.colorbar()
    plt.show()
    plt.savefig(outname)
    plt.clf()

  plot(vrad31, "rad31.png")
  plot(vbt31, "bt31.png")


